from http.cookiejar import Cookie
import subprocess
import sys

from PyQt5 import QtWidgets
from PyQt5 import QtGui
from PyQt5.QtCore import QUrl
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineScript

from onlinejudge import dispatch
from onlinejudge._implementation.utils import (
    default_cookie_path,
    with_cookiejar,
    new_session_with_our_user_agent,
)
from PyQt5.QtCore import QThread
from PyQt5.QtNetwork import QNetworkCookie

# Many of source is copied from https://qiita.com/montblanc18/items/88d0b639de86b7cac613


navi_script = """// ==UserScript==
// @match        https://atcoder.jp/contests/*
// ==/UserScript==
(function() {
    'use strict';
    if (!location.href.match(/^https:\/\/atcoder\.jp\/contests\/([^\/]+)/)) {
        return;
    }
    const contest_name = location.href.match(/^https:\/\/atcoder\.jp\/contests\/([^\/]+)/)[1];
    const key = 'atcoder-optimizer-' + contest_name;
     if (location.href.match(/^https:\/\/atcoder\.jp\/contests\/([^\/]+)\/tasks\/?$/)) {
        const tasks = [];
        let row;
        for (row of document.querySelectorAll('tbody > tr')) {
            const task = row.querySelectorAll('a');
            const href = task[0].getAttribute('href');
            const task_number = task[0].textContent;
            const task_name = task[1].textContent;
            tasks.push({
                href: href,
                task_name: task_number + ' - ' + task_name
            });
        }
        localStorage[key] = JSON.stringify(tasks);
    }
     if (key in localStorage) {
        var dom_obj = document.getElementById("main-container");
        dom_obj.style.paddingTop = "0px";

        if (document.getElementById("sourceCode")) {
           var dom_obj = document.getElementById("sourceCode");
           var dom_obj_parent = dom_obj.parentNode.parentNode.parentNode;
           dom_obj_parent.removeChild(dom_obj.parentNode.parentNode);

           var dom_obj = document.getElementsByTagName("footer")[0];
           var dom_obj_parent = dom_obj.parentNode.parentNode;
           dom_obj_parent.removeChild(dom_obj.parentNode);
        }

        const tasksBar = document.createElement('l');
        tasksBar.className = 'nav nav-tabs';
        let task;
        for (task of JSON.parse(localStorage[key])) {
            const link = document.createElement('a');
            link.setAttribute('style', 'margin-right: 20px;');
            link.setAttribute('href', task.href);
            link.textContent = task.task_name;
            tasksBar.appendChild(link);
        }
        document.getElementById('contest-nav-tabs').innerHTML = '';
        document.getElementById('contest-nav-tabs').appendChild(tasksBar);
    }
})();"""


yukicoder_script = """// ==UserScript==
// @match        https://yukicoder.me/problems/*
// ==/UserScript==
(function() {
    'use strict';
    var dom_obj = document.getElementById("sidebar");
    var dom_obj_parent = dom_obj.parentNode;
    dom_obj_parent.removeChild(dom_obj);
})();"""


class CustomWebEngineView(QWebEngineView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.runScript()
        self.setZoomFactor(0.97)

    def contextMenuEvent(self, a0: QtGui.QContextMenuEvent) -> None:
        menu = QtWidgets.QMenu()
        if self.parent().next_prev_updater.next is not None:
            menu.addAction(u"Go Next Task", self.parent().goNextTask)
        if self.parent().next_prev_updater.prev is not None:
            menu.addAction(u"Go Previous Task", self.parent().goPreviousTask)
        if self.selectedText():
            menu.addAction(u"View Graph", self.viewGraph)
        menu.addAction(u"Back", self.back)
        menu.addSeparator()
        if self.parent().next_prev_updater.problems is not None:
            problems = self.parent().next_prev_updater.problems
            for i, problem_url in enumerate(problems):
                problem_id = problem_url.split("/")[-1]
                # I know this is not good code.
                # But commented out code does not works
                # menu.addAction(
                #     u"Go to {}".format(problem_id),
                #     lambda: self.parent().changePage(problem)
                # )
                if i == 0:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[0]),
                    )
                elif i == 1:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[1]),
                    )
                elif i == 2:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[2]),
                    )
                elif i == 3:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[3]),
                    )
                elif i == 4:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[4]),
                    )
                elif i == 5:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[5]),
                    )
                elif i == 6:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[6]),
                    )
                elif i == 7:
                    menu.addAction(
                        u"Go to {}".format(problem_id),
                        lambda: self.parent().changePage(problems[7]),
                    )
        menu.exec(a0.globalPos())

    def runScript(self) -> None:
        # self.page().runJavaScript(navi_scripts, QWebEngineScript.ApplicationWorld)
        script = QWebEngineScript()
        script.setName("atcoder-optimizer")
        script.setSourceCode(navi_script)
        script.setInjectionPoint(QWebEngineScript.DocumentReady)
        script.setRunsOnSubFrames(True)
        script.setWorldId(QWebEngineScript.ApplicationWorld)
        self.page().scripts().insert(script)

    def viewGraph(self) -> None:
        text = self.selectedText()
        graph_member = []
        weighted = "false"

        lines = text.split("\n")
        vertexes = []
        for line in lines:
            words = line.split(" ")
            if len(words) < 2:
                continue
            if len(words) == 3:
                weighted = "true"
            graph_member.append("-".join(words))
            vertexes.append(int(words[0]))
            vertexes.append(int(words[1]))

        if min(vertexes) == 0:
            indexed = "false"
        else:
            indexed = "true"

        max_vertex = max(vertexes)
        if indexed == "false":
            max_vertex -= 1
        graph_member = ["{0}-{1}".format(max_vertex, len(lines))] + graph_member
        graph_query = ",".join(graph_member)

        url = "https://hello-world-494ec.firebaseapp.com/?format=true&directed=false&weighted={0}&indexed={1}&data={2}".format(
            weighted, indexed, graph_query
        )
        try:
            subprocess.Popen(["sensible-browser", url])
        except subprocess.TimeoutExpired:
            pass


class WebViewWindow(QtWidgets.QWidget):
    def __init__(self, parent=None, *args):
        super().__init__(parent, *args)
        self.browser = CustomWebEngineView(self)
        self.browser.resize(1000, 600)
        self.browser.setWindowTitle("Task")
        self.url_edit = QtWidgets.QLineEdit()
        self.url_edit.returnPressed.connect(self.loadPage)
        session = new_session_with_our_user_agent()
        self.browser.page().profile().setHttpUserAgent(session.headers["User-Agent"])
        self.browser.page().profile().cookieStore().cookieAdded.connect(
            self.handleCookieAdded
        )
        self.browser.urlChanged.connect(self.download_task)

        grid = QtWidgets.QGridLayout()
        grid.addWidget(self.url_edit, 0, 0, 1, 15)
        grid.addWidget(self.browser, 1, 0, 5, 15)
        self.setLayout(grid)
        self.resize(800, 800)
        self.next_prev_updater = NextPreviousProblemUpdater()

    def download_task(self):
        self.next_prev_updater.url = self.browser.url().toString()
        self.next_prev_updater.start()
        self.url_edit.setText(self.browser.url().toString())
        self.parent().parent().download(self.url_edit.text())

    def loadPage(self):
        self.next_prev_updater.reset()
        if self.url_edit.text():
            self.browser.load(QUrl(self.url_edit.text()))

    def changePage(self, url: str):
        self.url_edit.setText(url)
        self.loadPage()

    def goNextTask(self):
        """ Go next contest task.
        ex.
        https://atcoder.jp/contests/cpsco2019-s2/tasks/cpsco2019_s2_e ->
        https://atcoder.jp/contests/cpsco2019-s2/tasks/cpsco2019_s2_f
        """
        if self.next_prev_updater.next is not None:
            self.changePage(self.next_prev_updater.next)

    def goPreviousTask(self):
        """ Go previous contest task.
        ex.
        https://atcoder.jp/contests/cpsco2019-s2/tasks/cpsco2019_s2_b ->
        https://atcoder.jp/contests/cpsco2019-s2/tasks/cpsco2019_s2_a
        """
        if self.next_prev_updater.prev is not None:
            self.changePage(self.next_prev_updater.prev)

    def focusOnUrlEdit(self):
        self.url_edit.setFocus()
        self.url_edit.selectAll()

    def handleCookieAdded(self, cookie: QNetworkCookie):
        url = self.browser.url().toString()
        if dispatch.service_from_url(url):
            py_cookie = toPyCookie(cookie)
            with with_cookiejar(
                new_session_with_our_user_agent(), path=default_cookie_path
            ) as sess:
                sess.cookies.set_cookie(py_cookie)


class NextPreviousProblemUpdater(QThread):
    def __init__(self):
        super().__init__()
        self.url = ""
        self.next = None
        self.prev = None
        self.problems = None

    def reset(self):
        self.next = None
        self.prev = None
        # self.problems = None

    def run(self):
        cur_problem = dispatch.problem_from_url(self.url)
        if cur_problem is None:
            return
        if cur_problem.get_service().get_name() == "yukicoder":
            words = self.url.split("?")
            words = words[0].split("/")
            prev_words = words.copy()
            prev_words[-1] = str(int(words[-1]) - 1)
            next_words = words.copy()
            next_words[-1] = str(int(words[-1]) + 1)
            self.prev = "/".join(prev_words)
            self.next = "/".join(next_words)
            return
        try:
            contest = cur_problem.get_contest()
            with with_cookiejar(
                new_session_with_our_user_agent(), path=default_cookie_path
            ) as sess:
                problems = contest.list_problems(session=sess)
            for i, problem in enumerate(problems):
                if problem == cur_problem:
                    self.prev = problems[i - 1].get_url()
                    self.next = problems[(i + 1) % len(problems)].get_url()
                    break
            self.problems = [problem_url.get_url() for problem_url in problems]
        except Exception:
            pass


def toPyCookie(qt_cookie: QNetworkCookie) -> Cookie:
    port = None
    port_specified = False
    secure = qt_cookie.isSecure()
    name = qt_cookie.name().data().decode()
    value = qt_cookie.value().data().decode()
    v = qt_cookie.path()
    path_specified = bool(v != "")
    path = v if path_specified else None
    v = qt_cookie.domain()
    domain_specified = bool(v != "")
    domain = v
    if domain_specified:
        domain_initial_dot = v.startswith(".")
    else:
        domain_initial_dot = None
    v = int(qt_cookie.expirationDate().toTime_t())
    expires = 2147483647 if v > 2147483647 else v
    rest = {"HttpOnly": qt_cookie.isHttpOnly()}
    discard = False
    return Cookie(
        0,
        name,
        value,
        port,
        port_specified,
        domain,
        domain_specified,
        domain_initial_dot,
        path,
        path_specified,
        secure,
        expires,
        discard,
        None,
        None,
        rest,
    )


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ex = WebViewWindow()
    ex.show()
    sys.exit(app.exec_())



import subprocess
import sys
import time

from onlinejudge._implementation.utils import (
    default_cookie_path,
    with_cookiejar,
    new_session_with_our_user_agent,
)
from onlinejudge.dispatch import submission_from_url
from PyQt5 import QtGui, QtCore, QtWidgets
from PyQt5.QtCore import Qt, QThread, pyqtSignal


class External(QThread):
    updateRequest = pyqtSignal(object)
    finishRequest = pyqtSignal()
    failed_words = ("WA", "RE", "TLE", "MLE", "RE", "CE")

    def __init__(self, url: str):
        super().__init__()
        self.url = url

    def run(self):
        finished = False
        with with_cookiejar(
            new_session_with_our_user_agent(), path=default_cookie_path
        ) as sess:
            for i in range(40):
                time.sleep(2)
                submission = submission_from_url(self.url)
                result = submission.download_data(session=sess)
                self.updateRequest.emit((result.status, result.problem_id))
                result = result.status
                failed = any([word in result for word in self.failed_words])
                if result == "AC" or failed:
                    finished = True
                if failed:
                    subprocess.check_call(
                        ["sensible-browser", submission.get_url()],
                        stdin=sys.stdin,
                        stdout=sys.stdout,
                        stderr=sys.stderr,
                    )
                    finished = True
                if finished:
                    time.sleep(3)
                    self.finishRequest.emit()
                    break
            time.sleep(3)
            self.finishRequest.emit()


class CustomPopup(QtWidgets.QWidget):
    def __init__(self, parent, url: str, **kwargs):
        super().__init__(parent)
        self.setWindowOpacity(0.7)
        # self.setWindowFlags(QtCore.Qt.Popup | QtCore.Qt.FramelessWindowHint)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.setWindowTitle("CustomPopup")

        # set style
        radius = 40.0
        path = QtGui.QPainterPath()
        self.resize(220, 90)
        path.addRoundedRect(QtCore.QRectF(self.rect()), radius, radius)
        mask = QtGui.QRegion(path.toFillPolygon().toPolygon())
        self.setMask(mask)

        # set background
        self.setAutoFillBackground(True)

        # set label
        self.label = QtWidgets.QLabel(" " * 50, self)
        self.move(QtGui.QCursor.pos())
        self.run(url)

    def run(self, url: str):
        self.ext = External(url)
        self.ext.updateRequest.connect(self.update)
        self.ext.finishRequest.connect(self.finish)
        self.ext.start()

    def update(self, result):
        result, problem_id = result
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        self.label.setFont(font)
        self.label.move(25, 35)

        print("submit result: " + problem_id + " " + result)
        self.label.setText(
            "<font color='white'>" + problem_id + ": " + result + "</font>"
        )

        if result == "AC":
            self.setStyleSheet("background-color: green")
        elif result == "WJ" or (
            "/" in result and "WA" not in result and "RE" not in result
        ):
            self.setStyleSheet("background-color: grey")
        else:
            self.setStyleSheet("background-color: orange")
        self.repaint()

    def finish(self):
        self.ext.exit(0)
        self.close()

    def mousePressEvent(self, event: QtGui.QMouseEvent):
        self.close()


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)

    w = CustomPopup(url="https://atcoder.jp/contests/diverta2019/submissions/5363629")
    w.show()
    sys.exit(app.exec_())



import os
import re
import subprocess
from typing import *

import pexpect
from PyQt5 import QtWidgets
from PyQt5.QtCore import QThread, QMutex


TEMPFILE = os.path.join(os.path.dirname(__file__), "temp.rs")
TEMPFILE_CPP = os.path.join(os.path.dirname(__file__), "temp.cpp")


def disp_error(message: str):
    error = QtWidgets.QErrorMessage()
    error.showMessage(message)
    error.exec_()


def racer_enable() -> bool:
    try:
        output = subprocess.check_output(("racer", "--version"))
        if output.decode().startswith("racer"):
            return True
        else:
            return False
    except subprocess.CalledProcessError:
        return False


def debug_command(lang: str) -> str:
    if lang == "rust":
        return "env RUST_BACKTRACE=1 rust-gdb"
    if lang == "python3":
        return "python3 -m pdb"
    else:
        return "gdb"


def compile_command(lang: str, no_debug: bool) -> List[str]:
    if lang == "rust":
        if no_debug:
            return ["rustc"]
        else:
            return ["rustc", "-g"]
    elif lang == "python3":
        return ["python3", "py_syntax_checker.py"]
    else:
        if no_debug:
            return [
                "g++",
                "-std=gnu++1y",
                "-O2",
                "-I/opt/boost/gcc/include",
                "-L/opt/boost/gcc/lib",
            ]
        else:
            return [
                "g++",
                "-std=gnu++1y",
                "-g",
                "-I/opt/boost/gcc/include",
                "-L/opt/boost/gcc/lib",
            ]


def get_compiled_file(lang: str, fname: str) -> str:
    if lang == "rust":
        return "./" + os.path.basename(fname.replace(".rs", ""))
    elif lang == "python3":
        return fname
    else:
        return "./a.out"


def exec_format(lang: str) -> bool:
    if lang == "rust":
        try:
            subprocess.check_output(("rustfmt", TEMPFILE), stderr=subprocess.STDOUT)
        except Exception:
            return False
    elif lang == "python3":
        try:
            subprocess.check_output(
                ("autopep8", "-i", TEMPFILE), stderr=subprocess.STDOUT
            )
        except Exception:
            return False
    else:
        try:
            subprocess.check_output(
                ("clang-format", "-i", TEMPFILE), stderr=subprocess.STDOUT
            )
        except Exception:
            return False
    return True


def exec_command(lang: str) -> List[str]:
    if lang == "rust":
        return ["env", "RUST_BACKTRACE=1"]
    elif lang == "python3":
        return ["python3"]
    else:
        return []


def indent_width(lang: str) -> int:
    if lang == "rust" or lang == "python3":
        return 4
    else:
        return 2


class StateLessTextEdit(QtWidgets.QLineEdit):
    def __init__(self, value: str, parent):
        super().__init__()
        self.parent = parent
        self.setText(value)


# Need to call commit to serialize value
class StateFullTextEdit(QtWidgets.QLineEdit):
    def __init__(self, settings, name: str, parent, default: Optional[str] = None):
        super().__init__()
        self.parent = parent
        self.settings = settings
        self.name = name
        v = settings.value(name, type=str)
        self.setText(v)
        if not v and default is not None:
            self.setText(default)

    def commit(self):
        self.settings.setValue(self.name, self.text())


class StateFullCheckBox(QtWidgets.QCheckBox):
    def __init__(self, settings, name: str, parent):
        super().__init__()
        self.parent = parent
        self.settings = settings
        self.name = name
        v = settings.value(name, type=bool)
        self.setChecked(v)

    def commit(self):
        self.settings.setValue(self.name, self.isChecked())


def get_resources_dir() -> str:
    return os.path.join(os.path.dirname(__file__), "resources")


def wait_input_ready(
    debug_process: pexpect.spawn, lang: str, timeout: Optional[float] = None
):
    if lang == "python3":
        debug_process.expect("\(Pdb\)", timeout=timeout)
    else:
        debug_process.expect("\(gdb\)", timeout=timeout)


def get_executing_line(lang: str, line: str) -> Optional[int]:
    if lang == "python3":
        if line.endswith("<module>()"):
            match = re.search(r"(\()(.*?)\)", line)
            return int(match.groups()[-1])
    else:
        try:
            line_num = int(line.split("\t")[0])
            return line_num
        except ValueError:
            return None
    return None


class WriteObj(object):
    def __init__(self, msg: Union[bytes, str], mode: str = ""):
        self.msg = msg
        self.mode = mode


OJ_MUTEX = QMutex()


class Commander(QThread):
    def __init__(self, console):
        super().__init__()
        self.console = console
        self.cmd = ""

    def run(self):
        OJ_MUTEX.lock()
        try:
            out = subprocess.check_output(self.cmd, stderr=subprocess.STDOUT).decode()
            self.console.writeLnSignal.emit(out)
        except subprocess.CalledProcessError as err:
            self.console.writeLnSignal.emit(err.output)
        scroll_bar = self.console.verticalScrollBar()
        scroll_bar.setValue(scroll_bar.maximum())
        OJ_MUTEX.unlock()



#!/usr/bin/env python3
import codecs
import glob
import os
import shutil
import subprocess
import sys
from typing import *

from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5.QtGui import QIcon
from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt
import pexpect

from rujaion import console
from rujaion import display_widget
from rujaion import editor
from rujaion.command import login, submit, test
from rujaion import util
from rujaion.util import WriteObj
from rujaion import webview_widget

# TODO: watch selected variable
# TODO: highlight multi line comment
# TODO: display tree
# TODO: highlight selecting variable
# TODO: progress bar
# TODO: rustsym(find usage)r
# TODO: highlight corresponding parenthesis


class RujaionMainWindow(QtWidgets.QMainWindow):
    test_data_dir = "./test"
    test_case_editor = "gedit"

    def __init__(self, parent=None):
        super(RujaionMainWindow, self).__init__(parent)
        self.settings = QtCore.QSettings("RustDebugger", "RustDebugger")

        self.setWindowTitle("Rujaion")
        self.setStyleSheet("background-color: white")

        # lang_box placed before add editor
        self.lang_box = QtWidgets.QComboBox(self)
        self.lang_box.addItem("rust")
        self.lang_box.addItem("c++")
        self.lang_box.addItem("python3")

        self.addCentral()

        self.file_tool = self.addToolBar("File")
        newbutton = self.file_tool.addAction("New...")
        newbutton.triggered.connect(self.newFile)

        openbutton = self.file_tool.addAction("Open...")
        openbutton.triggered.connect(self.showFileDialog)

        savebutton = self.file_tool.addAction("Save...")
        savebutton.triggered.connect(self.saveFile)

        # compilebutton = self.edit_tool.addAction("Compile...")
        # compilebutton.triggered.connect(self.compile)

        self.contest_tool = self.addToolBar("Contest")
        testbutton = self.contest_tool.addAction("Test...")
        testbutton.triggered.connect(self.testMyCode)

        debugbutton = self.contest_tool.addAction("Debug TestCase...")
        debugbutton.triggered.connect(self.debugWithTestData)

        submitbutton = self.contest_tool.addAction("Submit!...")
        submitbutton.triggered.connect(self.submit)

        self.debug_tool = self.addToolBar("Debug")
        continuebutton = self.debug_tool.addAction("Continue...")
        continuebutton.triggered.connect(self.continue_process)

        nextbutton = self.debug_tool.addAction("Next...")
        nextbutton.triggered.connect(self.next)

        stepinbutton = self.debug_tool.addAction("Step In...")
        stepinbutton.triggered.connect(self.stepIn)

        stepoutbutton = self.debug_tool.addAction("Step Out...")
        stepoutbutton.triggered.connect(self.stepOut)

        terminatebutton = self.debug_tool.addAction("Terminate...")
        terminatebutton.triggered.connect(self.terminate)

        findbutton = self.debug_tool.addAction("Find...")
        findbutton.triggered.connect(self.editor.find)

        self.config_tool = self.addToolBar("Config")
        self.config_tool.addWidget(self.lang_box)

        # Add MenuBar
        menuBar = self.menuBar()
        filemenu = menuBar.addMenu("&File")

        a = QtWidgets.QAction("New", self)
        a.triggered.connect(self.newFile)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Open", self)
        a.triggered.connect(self.showFileDialog)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Save", self)
        a.triggered.connect(self.saveFile)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Save as", self)
        a.triggered.connect(self.saveFileAs)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Clear Backup", self)
        a.triggered.connect(self.clearBackup)
        filemenu.addAction(a)

        filemenu = menuBar.addMenu("&Program")

        a = QtWidgets.QAction("Run (Ctrl+F9)", self)
        a.triggered.connect(self.run)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Debug (F9)", self)
        a.triggered.connect(self.debug)
        filemenu.addAction(a)

        filemenu = menuBar.addMenu("&Contest")

        a = QtWidgets.QAction("Login", self)
        a.triggered.connect(self.login)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Test My Code (Ctrl+F4)", self)
        a.triggered.connect(self.testMyCode)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Test With Options", self)
        a.triggered.connect(self.testMyCodeWithOptions)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Debug With Test Data (F4)", self)
        a.triggered.connect(self.debugWithTestData)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Add Test", self)
        a.triggered.connect(self.addTest)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Clear Test Data", self)
        a.triggered.connect(self.clearTestData)
        filemenu.addAction(a)

        a = QtWidgets.QAction("Submit!", self)
        a.triggered.connect(self.submit)
        filemenu.addAction(a)

        self.debug_process = None
        try:
            self.openFile(self.settings.value("LastOpenedFile", type=str))
        except FileNotFoundError:
            pass
        self.lang_box.setCurrentText(self.editor.lang)
        self.lang_box.currentTextChanged.connect(self.editor.lang_changed)
        self.resize(self.settings.value("size", QtCore.QSize(1000, 900)))
        self.move(self.settings.value("pos", QtCore.QPoint(50, 50)))
        self.last_used_testcase = ""
        self.gdb_timeout = 4.0
        self.browser_dock = QtWidgets.QDockWidget("", self)
        self.addDockWidget(Qt.RightDockWidgetArea, self.browser_dock)
        self.addConsole()
        self.commander = util.Commander(self.console)
        self.addDisplay()
        self.addBrowser()

    def updateWindowTitle(self, running: bool = False):
        title = ""
        if self.editor.edited:
            title = "(*) "
        if self.debug_process is not None:
            title += "(Debugging...) "
        elif running:
            title += "(Running...) "
        if self.editor.fname:
            title += self.editor.fname
        else:
            title += "Scratch"
        self.setWindowTitle(title)

    def closeEvent(self, e: QtGui.QKeyEvent):
        self.settings.setValue("size", self.size())
        self.settings.setValue("pos", self.pos())
        e.accept()

    def keyPressEvent(self, event: QtGui.QKeyEvent):
        if (
            event.modifiers() == QtCore.Qt.ShiftModifier
            and event.key() == QtCore.Qt.Key_F8
        ):
            self.stepOut()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_F9
        ):
            self.run()
        elif event.key() == QtCore.Qt.Key_F9:
            self.debug()
        elif event.key() == QtCore.Qt.Key_F8:
            self.next()
        elif event.key() == QtCore.Qt.Key_F7:
            self.stepIn()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_F4
        ):
            self.testMyCode()
        elif event.key() == QtCore.Qt.Key_F4:
            self.debugWithLastCase()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_F6
        ):
            self.submit()
        elif event.key() == QtCore.Qt.Key_F6:
            self.browser_widget.focusOnUrlEdit()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_F12
        ):
            if self.debug_process is not None:
                return

            if self.editor.isHidden():
                self.editor.show()
                self.console.show()
                self.browser_dock.hide()
            else:
                self.editor.hide()
                self.console.hide()
                self.browser_dock.show()
                self.editor.setFocus()
        elif event.key() == QtCore.Qt.Key_F12:
            if self.editor.isHidden():
                self.editor.show()
                self.console.show()
                self.browser_widget.browser.setFocus()
            if self.browser_dock.isHidden():
                self.browser_dock.show()
            else:
                self.browser_dock.hide()
        elif event.key() == QtCore.Qt.Key_F11:
            if self.console_dock.isHidden():
                self.console_dock.show()
            else:
                self.console_dock.hide()
        elif event.key() == QtCore.Qt.Key_Escape:
            self.terminate()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_O
        ):
            self.showFileDialog()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_S
        ):
            self.saveFile()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_W
        ):
            self.saveFileAs()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_F
        ):
            self.editor.find()
        elif (
            event.modifiers() == QtCore.Qt.ControlModifier
            and event.key() == QtCore.Qt.Key_N
        ):
            self.newFile()
        else:
            event.accept()

    def showFileDialog(self):
        dirname = os.path.dirname(self.settings.value("LastOpenedFile", type=str))
        fname = QtWidgets.QFileDialog.getOpenFileName(
            self, "Open", dirname, "Source Files (*.rs *.cpp *.cc *.py *.bak)"
        )[0]
        self.openFile(fname)

    def openFile(self, fname: str):
        if not fname:
            return
        self.editor.open_file(fname)
        self.settings.setValue("LastOpenedFile", fname)
        self.updateWindowTitle()

    def clearBackup(self):
        if not self.editor.fname:
            return
        for fname in glob.glob(os.path.dirname(self.editor.fname) + "/*.bak"):
            os.remove(fname)

    def askTerminateOrNot(self) -> bool:
        ret = QtWidgets.QMessageBox.information(
            None,
            "Debugging process is runnning",
            "Do you want to terminate debug process" " and start new process?",
            QtWidgets.QMessageBox.Yes,
            QtWidgets.QMessageBox.No,
        )

        if ret == QtWidgets.QMessageBox.Yes:
            return True
        else:
            return False

    def saveFileAs(self):
        if self.editor.fname:
            default_dir = os.path.dirname(self.editor.fname)
        else:
            default_dir = os.path.dirname(
                self.settings.value("LastOpenedFile", type=str)
            )
        default_file_name = os.path.join(default_dir, self.editor.default_file_name())
        savename = QtWidgets.QFileDialog.getSaveFileName(
            self, "Save file", default_file_name, "Source Files (*.rs *.cpp)"
        )[0]
        if not savename:
            return
        self.editor.save_pre_process()
        with codecs.open(savename, "w", "utf-8") as f:
            f.write(self.editor.toPlainText())
        self.editor.fname = savename
        self.editor.reset_lang()
        self.savePostProcess()
        self.settings.setValue("LastOpenedFile", savename)

    def savePostProcess(self):
        self.updateWindowTitle()
        self.compile()

    def saveFile(self):
        if self.editor.fname:
            if self.editor.edited and os.path.exists(self.editor.fname):
                for i in range(50):
                    backup_name = self.editor.fname + "." + str(i) + ".bak"
                    if not os.path.exists(backup_name):
                        shutil.copy(self.editor.fname, backup_name)
                        break
            self.editor.save_pre_process()
            with codecs.open(self.editor.fname, "w", "utf-8") as f:
                f.write(self.editor.toPlainText())
            self.savePostProcess()
        else:
            self.saveFileAs()

    def addEditer(self, parent):
        self.editor = editor.Editter(parent)
        self.editor.toggleBreakSignal.connect(self.UpdateBreak)
        self.edited = False
        self.editor.textChanged.connect(self.updateWindowTitle)

    def addCentral(self):
        self.addEditer(self)
        self.setCentralWidget(self.editor)

    def addDisplay(self):
        self.display_widget = display_widget.ResultTableModel(self)
        self.browser_dock.setWidget(self.display_widget)
        self.display_widget.cellChanged.connect(self.processDisplayEdited)

    def addBrowser(self):
        self.browser_widget = webview_widget.WebViewWindow(self)
        self.browser_dock.setWidget(self.browser_widget)
        url = self.settings.value("contest url", "")
        self.browser_widget.changePage(url)

    def processDisplayEdited(self, row_num, column_num):
        if self.debug_process is not None:
            self.display_one_valuable(row_num)

    def addConsole(self):
        self.console = console.Console(self)
        self.console_dock = QtWidgets.QDockWidget("Console", self)
        self.console_dock.setWidget(self.console)
        self.addDockWidget(Qt.BottomDockWidgetArea, self.console_dock)

    def newFile(self):
        if self.lang_box.currentText() == "rust":
            self.editor.new_file(os.path.join(util.get_resources_dir(), "template.rs"))
        elif self.lang_box.currentText() == "c++":
            self.editor.new_file(os.path.join(util.get_resources_dir(), "template.cpp"))
        else:
            self.editor.new_file(os.path.join(util.get_resources_dir(), "template.py"))

    def compile(self, no_debug: bool = False):
        self.console.clear()
        self.editor.reset_compile_info()
        if not self.editor.fname:
            util.disp_error("File is not opened.")
        command = util.compile_command(self.editor.lang, no_debug) + [self.editor.fname]
        try:
            out = subprocess.check_output(command, stderr=subprocess.STDOUT)
            self.console.writeLnSignal.emit("[+] Compile is finished successfully!")
            error_places, warning_places = self.parse_compile_error(out.decode())
            self.editor.highlight_compile_error(warning_places, is_warning=True)
        except subprocess.CalledProcessError as err:
            self.console.writeSignal.emit(WriteObj(err.output, mode="error"))
            error_places, warning_places = self.parse_compile_error(err.output.decode())
            self.editor.highlight_compile_error(error_places, is_warning=False)
            return False
        return True

    def is_error_disp_line(self, lines: List[str], i: int):
        if self.editor.lang == "rust":
            if i == 0:
                return False
            return lines[i - 1].startswith("error") and "-->" in lines[i]
        elif self.editor.lang == "c++":
            return ": error:" in lines[i]
        else:
            return "error" in lines[i]

    def is_warning_disp_line(self, lines: List[str], i: int):
        if self.editor.lang == "rust":
            if i == 0:
                return False
            return lines[i - 1].startswith("warning") and "-->" in lines[i]
        else:
            return ": warning:" in lines[i]

    def parse_compile_error(self, error_message: str):
        error_disp_lines = []
        warning_disp_lines = []
        lines = error_message.split("\n")
        for i, line in enumerate(lines):
            if self.is_error_disp_line(lines, i):
                error_disp_lines.append(i)
            elif self.is_warning_disp_line(lines, i):
                warning_disp_lines.append(i)

        error_places = []
        for num in error_disp_lines:
            invalid_line, invalid_pos = lines[num].split(":")[1:3]
            error_places.append((int(invalid_line), int(invalid_pos)))

        warning_places = []
        for num in warning_disp_lines:
            invalid_line, invalid_pos = lines[num].split(":")[1:3]
            warning_places.append((int(invalid_line), int(invalid_pos)))

        return error_places, warning_places

    def run(self):
        if self.debug_process is not None:
            if self.askTerminateOrNot():
                self.terminate()
            else:
                return
        if not self.compile(no_debug=True):
            return
        self.updateWindowTitle(True)
        compiled_file = util.get_compiled_file(self.editor.lang, self.editor.fname)
        if not os.path.isfile(compiled_file):
            util.disp_error("Compiled file is not found.")
        try:
            output = subprocess.check_output(
                util.exec_command(self.editor.lang) + [compiled_file],
                stderr=subprocess.STDOUT,
            )
            self.console.writeLnSignal.emit(output)
        except subprocess.CalledProcessError as err:
            self.console.writeLnSignal.emit("[-] " + err.output.decode())
        self.console.writeLnSignal.emit("[+] Run process is finished successfully!")
        self.updateWindowTitle(False)

    def with_debug_display(func):
        def wrapper(self, *args, **kwargs):
            func(self, *args, **kwargs)
            if self.debug_process is not None:
                self.browser_dock.show()
                self.browser_dock.setWidget(self.display_widget)

        return wrapper

    def with_console(func):
        def wrapper(self, *args, **kwargs):
            func(self, *args, **kwargs)
            if self.debug_process is not None:
                self.console_dock.show()

        return wrapper

    @with_debug_display
    @with_console
    def debug(self, *args, **kwargs):
        self.console.clear()
        if not self.compile():
            return
        if self.debug_process is not None:
            if self.askTerminateOrNot():
                self.terminate()
            else:
                return
        compiled_file = util.get_compiled_file(self.editor.lang, self.editor.fname)
        if not os.path.isfile(compiled_file):
            util.disp_error("Compiled file is not found.")
        try:
            assert self.debug_process is None
            self.debug_process = pexpect.spawn(
                util.debug_command(self.editor.lang) + " " + compiled_file
            )
            util.wait_input_ready(self.debug_process, self.editor.lang)
            self.console.writeSignal.emit(WriteObj(self.debug_process.before.decode()))

            for com in self.editor.generateBreak():
                self.debug_process.send(com)
                util.wait_input_ready(self.debug_process, self.editor.lang)
                self.console.writeSignal.emit(
                    WriteObj(self.debug_process.before.decode(), mode="gdb")
                )

            print("run " + compiled_file)
            self.debug_process.send(b"run\n")
            self.updateWindowTitle()
            if self.editor.lang == "python3":
                util.wait_input_ready(self.debug_process, self.editor.lang)
                self.debug_process.send(b"continue\n")
            self.post_process()

        except subprocess.CalledProcessError as err:
            self.console.writeSignal(WriteObj(err, mode="error"))

    def debugWithLastCase(self):
        self.debugWithTestData(True)

    @with_debug_display
    @with_console
    def debugWithTestData(self, use_lastcase=False):
        self.console.clear()
        if not self.compile():
            return
        if self.debug_process is not None:
            if self.askTerminateOrNot():
                self.terminate()
            else:
                return
        if (
            use_lastcase
            and self.last_used_testcase
            and os.path.isfile(self.last_used_testcase)
        ):
            fname = self.last_used_testcase
        else:
            fname = QtWidgets.QFileDialog.getOpenFileName(
                self, "Open", self.test_data_dir, "Test data (*.in)"
            )[0]
        if not fname:
            self.last_used_testcase = ""
            return
        self.last_used_testcase = fname

        with open(fname) as fr:
            inputs = [line for line in fr if line]

        compiled_file = util.get_compiled_file(self.editor.lang, self.editor.fname)
        if not os.path.isfile(compiled_file):
            util.disp_error("Compiled file is not found.")
        try:
            if self.debug_process is None:
                self.debug_process = pexpect.spawn(
                    util.debug_command(self.editor.lang) + " " + compiled_file
                )
                self.console.terminate_evcxr()
            else:
                self.continue_process()
                return
            util.wait_input_ready(self.debug_process, self.editor.lang)
            for com in self.editor.generateBreak():
                self.debug_process.send(com)
                util.wait_input_ready(self.debug_process, self.editor.lang)
                self.console.writeSignal.emit(
                    WriteObj(self.debug_process.before.decode(), mode="gdb")
                )

            print("run " + compiled_file)
            self.debug_process.send(b"run\n")
            self.updateWindowTitle()
            if self.editor.lang == "python3":
                util.wait_input_ready(self.debug_process, self.editor.lang)
                self.debug_process.send(b"continue\n")
            for i, debug_input in enumerate(inputs):
                msg = self.debug_process.before.decode()
                for line in msg.split("\r\n"):
                    self.console.writeSignal.emit(WriteObj(line, mode="gdb"))

                for line in reversed(msg.split("\r\n")):
                    if line.endswith("exited normally]"):
                        if i != len(inputs) - 1:
                            self.console.writeLnSignal.emit(
                                "[-] Partial input is rejected"
                            )
                        self.terminate()
                        return
                    if "exited with code" in line:
                        self.console.writeLnSignal.emit(
                            "[-] Process is finished with error"
                        )
                        self.terminate()
                        return

                self.debug_process.send(debug_input.encode())
            self.post_process()
            self.updateWindowTitle()
            self.console.run_evcxr()

        except subprocess.CalledProcessError as err:
            self.console.writeSignal.emit(WriteObj(err, mode="error"))
            self.console.run_evcxr()

    def addTest(self):
        if not os.path.exists(self.test_data_dir):
            os.mkdir(self.test_data_dir)
        test_data_files = set(os.listdir(self.test_data_dir))
        for i in range(1, 100):
            if not "sample-{}.in".format(i) in test_data_files:
                in_file = os.path.join(self.test_data_dir, "sample-{}.in".format(i))
                out_file = os.path.join(self.test_data_dir, "sample-{}.out".format(i))
                subprocess.Popen(
                    ("gedit", in_file, out_file),
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                )
                break

    def UpdateBreak(self, command: bytes):
        if self.debug_process is None:
            return
        if command.startswith(b"b "):
            self.debug_process.send(command)
            util.wait_input_ready(self.debug_process, self.editor.lang)
        else:
            self.debug_process.send("i b\n".encode())
            util.wait_input_ready(self.debug_process, self.editor.lang)
            print(self.debug_process.before.decode())
            last_num = -1
            for line in self.debug_process.before.decode().split("\r\n"):
                if line.split(" ")[0].isdecimal():
                    last_num = int(line.split(" ")[0])
                if line.rstrip("\n").endswith(":" + command.decode()):
                    assert last_num != -1
                    self.debug_process.send(("d " + str(last_num) + "\n").encode())
                    util.wait_input_ready(self.debug_process, self.editor.lang)
                    break

    def next(self):
        print("next")
        if self.debug_process is None:
            return
        self.debug_process.send(b"n\n")
        self.post_process()

    def stepIn(self):
        print("step in")
        if self.debug_process is None:
            return
        self.debug_process.send(b"s\n")
        self.post_process()

    def stepOut(self):
        print("step out")
        if self.debug_process is None:
            return
        self.debug_process.send(b"fin\n")
        self.post_process()

    def terminate(self):
        print("quit")
        if self.debug_process is None:
            return
        self.debug_process.send(b"quit\n")
        self.debug_process.terminate()
        self.debug_process = None
        self.console.writeLnSignal.emit(
            "[+] Debug process was successfully terminated."
        )
        self.editor.clear_highlight_line()
        self.updateWindowTitle()
        self.browser_dock.setWidget(self.browser_widget)

    def continue_process(self):
        print("continue")
        if self.debug_process is None:
            return
        self.debug_process.send(b"c\n")
        self.post_process()

    def post_process(self):
        assert self.debug_process is not None
        try:
            util.wait_input_ready(
                self.debug_process, self.editor.lang, self.gdb_timeout
            )
        except:
            print(str(self.debug_process))
            self.console.writeLnSignal.emit("[-] Debug process is timeout")
            self.terminate()
            return
        msg = self.debug_process.before.decode()
        for line in msg.split("\r\n"):
            self.console.writeSignal.emit(WriteObj(line, mode="gdb"))

        for line in reversed(msg.split("\r\n")):
            if line.endswith("exited normally]") or line.startswith(
                "The program finished"
            ):
                self.terminate()
                return
            elif line.endswith("No such file or directory.") or line.endswith(
                "そのようなファイルやディレクトリはありません."
            ):
                self.stepOut()
                return
            elif "exited with code" in line:
                self.console.writeLnSignal.emit("[-] Process is finished with error")
                self.terminate()
                return
            else:
                line_num = util.get_executing_line(self.editor.lang, line)
                if line_num is None:
                    continue
                self.editor.highlight_executing_line(line_num)
                self.editor.repaint()
                break

        for i, _ in self.display_widget.name_iter():
            self.display_one_valuable(i)

    def display_one_valuable(self, row_num: int):
        # Avoid infinity loop
        self.display_widget.cellChanged.disconnect(self.processDisplayEdited)

        name = self.display_widget.item(row_num, 0).text()
        if not name:
            self.display_widget.set_cell(row_num, 1, "")
            self.display_widget.set_cell(row_num, 2, "")
            self.display_widget.cellChanged.connect(self.processDisplayEdited)
            return
        self.debug_process.send(b"p " + name.encode() + b"\n")
        util.wait_input_ready(self.debug_process, self.editor.lang)
        value = "".join(self.debug_process.before.decode().split("\r\n")[1:])
        value = value.split(" = ")[-1]
        # value = ''.join(value.split(' = ')[1:])
        self.display_widget.set_cell(row_num, 1, value)

        if self.editor.lang == "python3":
            self.debug_process.send(b"whatis " + name.encode() + b"\n")
        else:
            self.debug_process.send(b"pt " + name.encode() + b"\n")
        util.wait_input_ready(self.debug_process, self.editor.lang)
        type = "".join(self.debug_process.before.decode().split("\n")[1:])
        type = type.split(" = ")[-1]
        type = type.split(" {")[0]
        self.display_widget.set_cell(row_num, 2, type)

        self.display_widget.cellChanged.connect(self.processDisplayEdited)

    @with_console
    def download(self, url: str = None):
        self.settings.setValue("contest url", url)
        self.clearTestData()
        self.commander.cmd = ("oj", "download", url)
        self.commander.start()
        # self.console.writeLnSignal.emit("[+] Downloaded Test data")

    def login(self):
        login.LoginDialog(
            self,
            url=self.browser_widget.browser.url().toString(),
            settings=self.settings,
        ).show()

    def clearTestData(self):
        if os.path.isdir(self.test_data_dir):
            shutil.rmtree(self.test_data_dir)

    def exists_float_output(self) -> bool:
        """ If "." character found in any *.out file, return True.
        """
        try:
            subprocess.check_output(
                "grep -F . {}/*.out".format(self.test_data_dir), shell=True
            )
        except subprocess.CalledProcessError:
            return False
        return True

    @with_console
    def testMyCode(self, *args):
        self.console.clear()
        if not self.compile(no_debug=True):
            return
        compiled_file = util.get_compiled_file(self.editor.lang, self.editor.fname)
        test_command = "{}".format(
            " ".join(util.exec_command(self.editor.lang) + [compiled_file])
        )
        try:
            command = ["oj", "test", "-c", test_command]
            print(command)
            if self.exists_float_output():
                error = 0.00000001
                command += ["-e", str(error)]
                self.console.writeLnSignal.emit("[.] Found float expectation")
                self.console.writeLnSignal.emit("[.] Allow {} error".format(error))
            # TODO: configurable timeout
            out = subprocess.check_output(
                command, stderr=subprocess.STDOUT, timeout=4.0
            ).decode()
            self.console.writeLnSignal.emit(out)
        except subprocess.TimeoutExpired as e:
            self.console.writeLnSignal.emit(e.output)
            self.console.writeLnSignal.emit("[-] Test is Timeout")
            return
        except subprocess.CalledProcessError as e:
            self.console.writeLnSignal.emit(e.output)
        except Exception as e:
            self.console.writeSignal.emit(WriteObj(e, mode="error"))
            return

    @with_console
    def testMyCodeWithOptions(self, *args):
        self.console.clear()
        if not self.compile(no_debug=True):
            return
        compiled_file = util.get_compiled_file(self.editor.lang, self.editor.fname)
        test.TestDialog(
            self, compiled_file=compiled_file, settings=self.settings
        ).show()

    @with_console
    def submit(self, *args):
        if not self.editor.fname:
            util.disp_error("Please save this file")
        submit.SubmitDialog(
            self,
            url=self.browser_widget.browser.url().toString(),
            lang=self.editor.lang,
        ).show()


def main():
    app = QtWidgets.QApplication(sys.argv)
    icon_path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), "../img/icon.png"
    )
    app.setWindowIcon(QIcon(icon_path))
    main = RujaionMainWindow()
    main.show()
    app.exec_()


if __name__ == "__main__":
    main()



import re
from typing import *

from PyQt5.QtCore import QRegExp, Qt
from PyQt5.QtGui import QFont, QSyntaxHighlighter, QTextCharFormat, QColor


class HighlighterBase(QSyntaxHighlighter):
    colors = [
        QColor(Qt.magenta).lighter(150),
        Qt.cyan,
        Qt.yellow,
        Qt.green,
        Qt.darkCyan,
        Qt.darkYellow,
    ]

    def update_levenshtein(self, words: List[str]):
        self.levensteign_rules.clear()
        if len(words) > len(self.colors):
            return
        for word, color in zip(words, self.colors):
            format = QTextCharFormat()
            format.setBackground(color)
            self.levensteign_rules.append(
                (QRegExp("\\b" + re.escape(word) + "\\b"), format)
            )

    def highlightBlock(self, text: str):
        for pattern, format in self.highlight_rules + self.levensteign_rules:
            expression = QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, format)
                index = expression.indexIn(text, index + length)
        self.setCurrentBlockState(0)


class RustHighlighter(HighlighterBase):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.highlight_rules = []
        function_format = QTextCharFormat()
        function_format.setFontItalic(True)
        function_format.setForeground(Qt.blue)
        self.highlight_rules.append(
            (QRegExp("\\b[A-Za-z][A-Za-z0-9_:<>\+ ]+(?=\\()"), function_format)
        )

        keyword_format = QTextCharFormat()
        keyword_format.setForeground(QColor(Qt.magenta).darker(150))
        keyword_format.setFontWeight(QFont.Bold)

        # fmt: off
        keyword_patterns = ("\\bi32\\b", "\\bi64\\b", "\\bi16\\b",
                            "\\bu16\\b", "\\bu32\\b", "\\bu64\\b",
                            "\\bf16\\b", "\\bf32\\b", "\\bf64\\b",
                            "\\busize\\b", "\\bchar\\b", "\\bbool\\b",
                            "\\blet\\b", "\\bmut\\b",
                            "\\bfn\\b", "\\bstruct\\b", "\\bpub\\b",
                            "\\buse\\b", "\\bimpl\\b",
                            "\\bconst\\b", "\\bstatic\\b", "\\bstr\\b",
                            "\\bString\\b", "\\breturn\\b", "\\bwhere\\b",
                            "\\benum\\b", "\\bmatch\\b", "\\btype\\b",
                            "\\bfor\\b", "\\bwhile\\b", "\\bbreak\\b",
                            "\\bVec\\b", "\\bcontinue\\b", "\\bloop\\b",
                            "\\bin\\b", "\\belse\\b", "\\bif\\b",
                            "\\bas\\b",
                            )
        # fmt: on

        self.highlight_rules += [
            (QRegExp(pattern), keyword_format) for pattern in keyword_patterns
        ]

        const_format = QTextCharFormat()
        const_format.setForeground(Qt.darkYellow)

        # fmt: off
        const_patterns = ("\\btrue\\b", "\\bfalse\\b", "\\bNone\\b",
                          "\\bSome\\b", "\\b[0-9\.\_]+\\b", "\\b[0-9\_]+usize\\b",
                          "\\b[0-9_]+u32\\b", "\\b[0-9_]+u64\\b", "\\b[0-9_]+u128\\b",
                          "\\b[0-9_]+i32\\b", "\\b[0-9_]+i64\\b", "\\b[0-9_]+i128\\b",
                          "\\b[0-9\._]+f32\\b", "\\b[0-9\._]+f64\\b",
                          )
        # fmt: on

        self.highlight_rules += [
            (QRegExp(pattern), const_format) for pattern in const_patterns
        ]

        self_format = QTextCharFormat()
        self_format.setForeground(Qt.darkRed)
        self.highlight_rules.append((QRegExp("\\bself\\b"), self_format))

        quotation_format = QTextCharFormat()
        quotation_format.setForeground(Qt.darkGreen)
        self.highlight_rules.append((QRegExp('".*"'), quotation_format))
        self.highlight_rules.append((QRegExp("'.*'"), quotation_format))

        macro_format = QTextCharFormat()
        macro_format.setFontItalic(True)
        macro_format.setForeground(Qt.darkCyan)
        self.highlight_rules.append((QRegExp("\\b[A-Za-z0-9_]+!"), macro_format))

        attribute_format = QTextCharFormat()
        attribute_format.setForeground(QColor(Qt.green).darker(140))
        self.highlight_rules.append((QRegExp("#[^\n]*"), attribute_format))

        line_comment_format = QTextCharFormat()
        line_comment_format.setForeground(Qt.darkGray)
        self.highlight_rules.append((QRegExp("//[^\n]*"), line_comment_format))

        self.levensteign_rules = []


class CppHighlighter(HighlighterBase):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.highlight_rules = []
        function_format = QTextCharFormat()
        function_format.setFontItalic(True)
        function_format.setForeground(Qt.blue)
        self.highlight_rules.append(
            (QRegExp("\\b[A-Za-z][A-Za-z0-9_:<>\+ ]+(?=\\()"), function_format)
        )

        keyword_format = QTextCharFormat()
        keyword_format.setForeground(QColor(Qt.magenta).darker(150))
        keyword_format.setFontWeight(QFont.Bold)

        # fmt: off
        keyword_patterns = ("\\bint\\b", "\\blong\\b", "\\bsize_t\\b",
                            "\\bchar\\b", "\\bbool\\b",
                            "\\bauto\\b", "\\bstruct\\b", "\\bswitch\\b",
                            "\\bconst\\b", "\\bstatic\\b", "\\bstr\\b",
                            "\\bString\\b", "\\breturn\\b", "\\busing\\b",
                            "\\benum\\b", "\\btypedef\\b", "\\bnamespace\\b",
                            "\\bfor\\b", "\\bwhile\\b", "\\bbreak\\b",
                            "\\bcontinue\\b", "\\belse\\b", "\\bif\\b",
                            )
        # fmt: on

        self.highlight_rules += [
            (QRegExp(pattern), keyword_format) for pattern in keyword_patterns
        ]

        const_format = QTextCharFormat()
        const_format.setForeground(Qt.darkYellow)

        # fmt: off
        const_patterns = ("\\btrue\\b", "\\bfalse\\b", "\\bnullptr\\b",
                          "\\bNULL\\b", "\\bSome\\b", "\\b[0-9\.\_]+\\b",
                          )
        # fmt: on

        self.highlight_rules += [
            (QRegExp(pattern), const_format) for pattern in const_patterns
        ]

        self_format = QTextCharFormat()
        self_format.setForeground(Qt.darkRed)
        self.highlight_rules.append((QRegExp("\\bthis\\b"), self_format))

        quotation_format = QTextCharFormat()
        quotation_format.setForeground(Qt.darkGreen)
        self.highlight_rules.append((QRegExp('".*"'), quotation_format))
        self.highlight_rules.append((QRegExp("'.*'"), quotation_format))

        attribute_format = QTextCharFormat()
        attribute_format.setForeground(QColor(Qt.green).darker(140))
        self.highlight_rules.append((QRegExp("#[^\n]*"), attribute_format))

        line_comment_format = QTextCharFormat()
        line_comment_format.setForeground(Qt.darkGray)
        self.highlight_rules.append((QRegExp("//[^\n]*"), line_comment_format))

        self.levensteign_rules = []


class PyHighlighter(HighlighterBase):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.highlight_rules = []
        function_format = QTextCharFormat()
        function_format.setFontItalic(True)
        function_format.setForeground(Qt.blue)
        self.highlight_rules.append((QRegExp("\\b[A-Za-z]+(?=\\()"), function_format))

        keyword_format = QTextCharFormat()
        keyword_format.setForeground(QColor(Qt.magenta).darker(150))
        keyword_format.setFontWeight(QFont.Bold)

        # fmt: off
        keyword_patterns = ("\\bdef\\b", "\\bclass\\b",
                            "\\bglobal\\b", "\\breturn\\b", "\\bimport\\b",
                            "\\bfor\\b", "\\bwhile\\b", "\\bbreak\\b",
                            "\\bcontinue\\b", "\\belse\\b", "\\bif\\b",
                            "\\belif\\b", "\\btry\\b", "\\bexcept\\b",
                            "\\bfinally\\b", "\\bas\\b", "\\band\\b",
                            "\\bor\\b", "\\bnot\\b", "\\bin\\b",
                            )
        # fmt: on

        self.highlight_rules += [
            (QRegExp(pattern), keyword_format) for pattern in keyword_patterns
        ]

        const_format = QTextCharFormat()
        const_format.setForeground(Qt.darkYellow)

        # fmt: off
        const_patterns = ("\\bTrue\\b", "\\bTalse\\b", "\\bNone\\b",
                          "\\b[0-9\.\_]+\\b",
                          )
        # fmt: on

        self.highlight_rules += [
            (QRegExp(pattern), const_format) for pattern in const_patterns
        ]

        self_format = QTextCharFormat()
        self_format.setForeground(Qt.darkRed)
        self.highlight_rules.append((QRegExp("\\bself\\b"), self_format))

        quotation_format = QTextCharFormat()
        quotation_format.setForeground(Qt.darkGreen)
        self.highlight_rules.append((QRegExp('".*"'), quotation_format))
        self.highlight_rules.append((QRegExp("'.*'"), quotation_format))

        line_comment_format = QTextCharFormat()
        line_comment_format.setForeground(Qt.darkGray)
        self.highlight_rules.append((QRegExp("#[^\n]*"), line_comment_format))

        self.levensteign_rules = []



import subprocess

from PyQt5 import QtWidgets

from rujaion import util


class TestDialog(QtWidgets.QDialog):
    def __init__(self, *args, compiled_file: str, settings):
        super().__init__(*args)
        self.console = self.parent().console
        self.compiled_file = compiled_file
        # self.is_interactive = util.StateFullCheckBox(settings, "TEST_INTERACTIVE", self)
        self.tle_edit = util.StateFullTextEdit(settings, "TEST_TLE", self, "2.0")
        self.mle_edit = util.StateFullTextEdit(settings, "TEST_MLE", self, "1024")
        self.float_error_edit = util.StateFullTextEdit(
            settings, "TEST_FLOAT_ERROR", self
        )
        self.dialogs = (
            ("Submit...", None),
            # ("Is Interactive", self.is_interactive),
            ("TLE", self.tle_edit),
            ("MLE", self.mle_edit),
            ("FLOAT ERROR", self.float_error_edit),
        )
        self.draw()

    def draw(self, *args):
        main_layout = QtWidgets.QVBoxLayout()
        for name, widget in self.dialogs:
            if not widget:
                l_widget = QtWidgets.QGroupBox(name)
                l_widget.setStyleSheet(
                    """
                QGroupBox {
                    color: white;
                    border: 1px solid gray;
                    border-radius: 9px;
                    margin-top: 0.5em;
                }
                QGroupBox::title {
                    color: white;
                    subcontrol-origin: margin;
                    left: 10px;
                    padding: 0 3px 0 3px;
                }
                """
                )
                l_widget.setFlat(False)
                section_layout = QtWidgets.QFormLayout()
                l_widget.setLayout(section_layout)
                main_layout.addWidget(l_widget)
            else:
                section_layout.addRow(name, widget)
        test_button = QtWidgets.QPushButton("Test")
        test_button.clicked.connect(self.test)
        main_layout.addWidget(test_button)
        self.setLayout(main_layout)

    def test(self):
        for name, widget in self.dialogs:
            try:
                widget.commit()
            except AttributeError:
                pass
        try:
            if self.is_interactive.isChecked():
                command = ["oj", "t/r", "-c", self.compiled_file]
            else:
                command = ["oj", "test", "-c", self.compiled_file]
                if self.float_error_edit.text():
                    command += ["-e", str(self.float_error_edit.text())]
                if self.tle_edit.text():
                    command += ["--tle", str(self.tle_edit.text())]
                if self.mle_edit.text():
                    command += ["--mle", str(self.mle_edit.text())]

            out = subprocess.check_output(
                command, stderr=subprocess.STDOUT, timeout=4.0
            ).decode()
            self.console.writeLnSignal.emit(out)
        except subprocess.TimeoutExpired as e:
            self.console.writeLnSignal.emit(e.output)
            self.console.writeLnSignal.emit("[-] Test is Timeout")
        except subprocess.CalledProcessError as e:
            self.console.writeLnSignal.emit(e.output)
        self.close()
